//
//  MBGeometry.h
//  MBMapKit
//
//  Copyright 2011 Mapbar Inc. All rights reserved.
//


#import <CoreGraphics/CoreGraphics.h>
#import <CoreLocation/CLLocation.h>

#import <UIKit/UIKit.h>

/** @struct MBCoordinateSpan
 *
 *  @brief  定义一块地图区域跨度的结构体。
 *  @note  
 *      使用这个结构体的差值可以暗示期望的地图缩放等级，越小的差值对应越高的缩放等级。
 */
typedef struct {
    /** 地图显示部分南-北的距离（以度数计算）。与经度距离不同,不同经度度的距离是不同的，每纬度总是大约111千米。 */
    CLLocationDegrees latitudeDelta;
    /** 地图显示部分东-西的距离（以度数计算）。一个经度的跨度根据当前经度值变化。例如 赤道的一经度跨越大约111千米，而极地约0千米。 */
    CLLocationDegrees longitudeDelta;
} MBCoordinateSpan;

/** @struct MBCoordinateRegion
 *
 *  @brief  定义地图将要现实部分的结构体。
 */
typedef struct {
    /** 范围的中心坐标 */
	CLLocationCoordinate2D center;
    /** 地图显示部分垂直与水平的跨度。跨度定义了当前地图视图对象的缩放等级。 */
	MBCoordinateSpan span;
} MBCoordinateRegion;

/**
 *
 *  @brief  使用指定的数值创建一个新的MBCoordinateSpan。
 *  @param  latitudeDelta   地图显示部分南-北的距离（以度数计算）。与经度距离不同,不同经度度的距离是不同的，一纬度总是大约111千米。
 *  @param  longitudeDelta  地图显示部分东-西的距离（以度数计算）。一个经度的跨度根据当前经度值变化。例如 赤道的一经度跨越大约111千米，而极地约0千米。
 *  @return 由指定差值生成的跨度。
 */
UIKIT_STATIC_INLINE MBCoordinateSpan MBCoordinateSpanMake(CLLocationDegrees latitudeDelta, CLLocationDegrees longitudeDelta)
{
    MBCoordinateSpan span;
    span.latitudeDelta = latitudeDelta;
    span.longitudeDelta = longitudeDelta;
    return span;
}

/**
 *
 *  @brief  使用指定的坐标和跨度创建一个新的MBCoordinateRegion。
 *  @param  centerCoordinate    范围的中心坐标。
 *  @param  span    地图显示部分垂直与水平的跨度。跨度定义了当前地图视图对象的缩放等级。
 *  @return 由指定值生成的范围。
 */
UIKIT_STATIC_INLINE MBCoordinateRegion MBCoordinateRegionMake(CLLocationCoordinate2D centerCoordinate, MBCoordinateSpan span)
{
	MBCoordinateRegion region;
	region.center = centerCoordinate;
    region.span = span;
	return region;
}

/** 
 *
 *  @brief  使用指定的坐标和距离值创建一个新的MBCoordinateRegion。
 *  @param  centerCoordinate    新范围的中心坐标。
 *  @param  latitudinalMeters   跨度北-南向的距离（以米计算）。
 *  @param  longitudinalMeters  跨度东-西向的距离（以米计算）。
 *  @return 由指定值生成的范围。
 */
 UIKIT_EXTERN MBCoordinateRegion MBCoordinateRegionMakeWithDistance(CLLocationCoordinate2D centerCoordinate, CLLocationDistance latitudinalMeters, CLLocationDistance longitudinalMeters);

/** @struct MBMapPoint
 *
 *  @brief  二维地图投影的点。
 *  @note   
 *      如果把球形的曲面投影到平面，就会得到一个经线平行的二维地图。这种地图常用来一下显示整个地球的表面。MBMapPoint数据结
 *      构表示这个二维地图中的一个点。
 *      \n地图点单位用于绘制 MBMapView 的内容，但是你不用关心这些单位。你主要使用地图点来简化运算，直接使用曲面的坐标值会比较
 *      复杂。通过转换为地图点，你可以在平面执行这些运算，这样做通常更简单，如果有需要还可以转回去。你可以使用 MBMapPointForCoordinate 
 *      和 MBCoordinateForMapPoint 函数在坐标点和地图点间相互转换。
 *      \n当保存地图相关的数据时，你需要保存坐标值（经度和纬度）而不是地图点。
 */
typedef struct {
    /** 点在地图中x轴的位置*/
    double x;
    /** 点在地图中y轴的位置*/
    double y;   
} MBMapPoint;

/** @struct MBMapSize
 *
 *  @brief  二维地图投影的尺寸。
 *  @note   
 *      如果把球形的曲面投影到平面，就会得到一个经线平行的二维地图。这种地图常用来一下显示整个地球的表面。MBMapSize数据结
 *      构表示这个二维地图中的水平和垂直距离。  
 */
typedef struct {
    /** 宽度值。这个值的单位为地图点。 */
    double width; 
    /** 高度值。这个值的单位为地图点。*/
    double height;  
} MBMapSize;

/** @struct MBMapRect
 *
 *  @brief  二维地图投影的矩形区域。
 *  @note   
 *      如果把球形的曲面投影到平面，就会得到一个经线平行的二维地图。这种地图常用来一下显示整个地球的表面。MBMapRect数据结
 *      构表示这个二维地图中的一个矩形区域。  
 */
typedef struct {
    /** 矩形的原点。  */
    MBMapPoint origin;
    /** 矩形的宽度和高度，以原点开始。 */
    MBMapSize size;
} MBMapRect;

/** @constant MBZoomScale
 *  @brief  地图的缩放因子。
 *  @note
 *      MBZoomScale 提供MBMapPoints和屏幕点之间的转换因子。当MBZoomScale等于1, 一个屏幕点等于一个MBMapPoint.  当
 *      MBZoomScale等于0.5,一个屏幕点等于两个MBMapPoints.
 */
typedef CGFloat MBZoomScale;

/** @constant MBMapSizeWorld
 *
 *  @brief  指定二维地图投影中世界的高度及宽度（地图点）。
 */
UIKIT_EXTERN const MBMapSize MBMapSizeWorld;

/** @constant MBMapRectWorld
 *
 *  @brief  表示二维地图投影中世界的地图矩形。
 *  @note
 *      这个矩形包含世界中的每个地图点。
 */
UIKIT_EXTERN const MBMapRect MBMapRectWorld;

/**
 *
 *  @brief  返回指定坐标对应的地图点数据结构。
 *  @param  coordinate  坐标的经度及纬度。
 *  @return 指定坐标在二维地图投影中对应的地图点。
 */
UIKIT_EXTERN MBMapPoint MBMapPointForCoordinate(CLLocationCoordinate2D coordinate);

/**
 *
 *  @brief  返回指定地图点对应的经度及纬度。
 *  @param  mapPoint    二维地图投影中的地图点。
 *  @return 指定地图点对应的坐标的经度及纬度。
 */
UIKIT_EXTERN CLLocationCoordinate2D MBCoordinateForMapPoint(MBMapPoint mapPoint);

/**
 *
 *  @brief  返回指定纬度下一个地图点表示的距离跨度。
 *  @param  latitude    用于返回值的经度。
 *  @return 一个地图点表示的距离（以米为单位）。
 *  @note  
 *      当经度靠近极地是地图点间的距离减少，在不同纬度下这种变化和经度不同。
 */
UIKIT_EXTERN CLLocationDistance MBMetersPerMapPointAtLatitude(CLLocationDegrees latitude);

/**
 *
 *  @brief  返回指定纬度下一米表示的地图点数量。
 *  @param  latitude    用于返回值的经度。
 *  @return 一米跨过的地图点数量。
 */
UIKIT_EXTERN double MBMapPointsPerMeterAtLatitude(CLLocationDegrees latitude);

/**
 *
 *  @brief  两个地图点间的距离。
 *  @param  a   第一个地图点。
 *  @param  b   第二个地图点。
 *  @return 指定地图点间的距离。
 *  @note
 *      这个距离表示球面两点间的实际距离，考虑了地球的曲度。
 */
UIKIT_EXTERN CLLocationDistance MBMetersBetweenMapPoints(MBMapPoint a, MBMapPoint b);

/** @constant MBMapRectNull
 *
 *  @brief  无效的地图矩形。
 *  @note
 *     当你需要指定一个无效的地图矩形是可以使用这个常量。
 */
UIKIT_EXTERN const MBMapRect MBMapRectNull;

/**
 *
 *  @brief  使用指定的值创建一个新的MBMapPoint。
 *  @param  x   地图中东-西轴的点。
 *  @param  y   地图中北-南轴的点。
 *  @return 由指定值生成的地图点。
 */
UIKIT_STATIC_INLINE MBMapPoint MBMapPointMake(double x, double y) {
    return (MBMapPoint){x, y};
}

/**
 *
 *  @brief  使用指定的值创建一个新的MBMapSize。
 *  @param  width   地图中东-西轴的距离。
 *  @param  height  地图中北-南轴的距离。
 *  @return 由指定值生成的地图尺寸。
 */
UIKIT_STATIC_INLINE MBMapSize MBMapSizeMake(double width, double height) {
    return (MBMapSize){width, height};
}

/**
 *
 *  @brief  使用指定的值创建一个新的MBMapRect。
 *  @param  x   用于原点的地图中东-西轴的点。
 *  @param  y   用于原点的地图中北-南轴的点。
 *  @param  width   地图中东-西轴的距离。
 *  @param  height  地图中北-南轴的距离。
 *  @return 由指定值生成的地图矩形。
 */
UIKIT_STATIC_INLINE MBMapRect MBMapRectMake(double x, double y, double width, double height) {
    return (MBMapRect){ MBMapPointMake(x, y), MBMapSizeMake(width, height) };
}

/**
 *
 *  @brief  返回指定矩形x轴的最小值。
 *  @param  rect    地图矩形.
 *  @return x轴的最小值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMinX(MBMapRect rect) {
    return rect.origin.x;
}

/**
 *
 *  @brief  返回指定矩形y轴的最小值。
 *  @param  rect    地图矩形.
 *  @return y轴的最小值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMinY(MBMapRect rect) {
    return rect.origin.y;
}

/**
 *
 *  @brief  返回指定矩形x轴的中点。
 *  @param  rect    地图矩形.
 *  @return x轴的中点值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMidX(MBMapRect rect) {
    return rect.origin.x + rect.size.width / 2.0;
}

/**
 *
 *  @brief  返回指定矩形y轴的中点。
 *  @param  rect    地图矩形.
 *  @return y轴的中点值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMidY(MBMapRect rect) {
    return rect.origin.y + rect.size.height / 2.0;
}

/**
 *
 *  @brief   返回指定矩形x轴的最大值。
 *  @param  rect    地图矩形.
 *  @return x轴的最大值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMaxX(MBMapRect rect) {
    return rect.origin.x + rect.size.width;
}

/**
 *
 *  @brief  返回指定矩形y轴的最大值。
 *  @param  rect    地图矩形.
 *  @return y轴的最大值。
 */
UIKIT_STATIC_INLINE double MBMapRectGetMaxY(MBMapRect rect) {
    return rect.origin.y + rect.size.height;
}

/**
 *
 *  @brief  返回指定矩形的宽度。
 *  @param  rect    地图矩形.
 *  @return 矩形的宽度。
 */
UIKIT_STATIC_INLINE double MBMapRectGetWidth(MBMapRect rect) {
    return rect.size.width;
}

/**
 *
 *  @brief  返回指定矩形的高度。
 *  @param  rect    地图矩形.
 *  @return 矩形的高度。
 */
UIKIT_STATIC_INLINE double MBMapRectGetHeight(MBMapRect rect) {
    return rect.size.height;
}

/**
 *
 *  @brief  返回指示两个地图点是否相同的布尔值。
 *  @param  point1  第一个地图点。
 *  @param  point2  第二个点。
 *  @return 如果两个点的x和y值均相等，返回YES;否则返回NO;
 */
UIKIT_STATIC_INLINE BOOL MBMapPointEqualToPoint(MBMapPoint point1, MBMapPoint point2) {
    return point1.x == point2.x && point1.y == point2.y;
}

/**
 *
 *  @brief  返回指示两个地图尺寸是否相同的布尔值。
 *  @param  size1  第一个地图尺寸。
 *  @param  size2  第二个地图尺寸。
 *  @return 如果两个点的高度和宽度均相等，返回YES;否则返回NO;
 */
UIKIT_STATIC_INLINE BOOL MBMapSizeEqualToSize(MBMapSize size1, MBMapSize size2) {
    return size1.width == size2.width && size1.height == size2.height;
}

/**
 *
 *  @brief  返回指示两个地图矩形是否相同的布尔值。
 *  @param  rect1  第一个地图矩形。
 *  @param  rect2  第二个地图矩形。
 *  @return 如果两个矩形完全相等，返回YES;否则返回NO;
 */
UIKIT_STATIC_INLINE BOOL MBMapRectEqualToRect(MBMapRect rect1, MBMapRect rect2) {
    return 
    MBMapPointEqualToPoint(rect1.origin, rect2.origin) &&
    MBMapSizeEqualToSize(rect1.size, rect2.size);
}

/**
 *
 *  @brief  返回指示指定的矩形是否无效的布尔值。
 *  @param  rect    地图矩形。
 *  @return point2  如果两个矩形无效，返回YES;否则返回NO;
 *  @note
 *      如果矩形的原点为无效值或者无穷大值则被视为无效。
 */
UIKIT_STATIC_INLINE BOOL MBMapRectIsNull(MBMapRect rect) {
    return isinf(rect.origin.x) || isinf(rect.origin.y);
}

/**
 *
 *  @brief  返回指示指定的矩形没有区域。
 *  @param  rect    地图矩形。
 *  @return  point2  如果两个矩形无效或者它的宽度或高度等于0，返回YES;否则返回NO;
 */
UIKIT_STATIC_INLINE BOOL MBMapRectIsEmpty(MBMapRect rect) {
    return MBMapRectIsNull(rect) || (rect.size.width == 0.0 && rect.size.height == 0.0);
}

/**
 *
 *  @brief  返回指定地图点的格式化字符串。
 *  @param  point    需要格式化的地图点。
 *  @return  包含地图点x和y坐标的格式化字符串。
 */
UIKIT_STATIC_INLINE NSString *MBStringFromMapPoint(MBMapPoint point) {
    return [NSString stringWithFormat:@"{%.1f, %.1f}", point.x, point.y];
}

/**
 *
 *  @brief  返回指定地图尺寸的格式化字符串。
 *  @param  size    需要格式化的地图尺寸。
 *  @return  包含地图尺寸宽度及高度的格式化字符串。
 */
UIKIT_STATIC_INLINE NSString *MBStringFromMapSize(MBMapSize size) {
    return [NSString stringWithFormat:@"{%.1f, %.1f}", size.width, size.height];
}

/**
 *
 *  @brief  返回指定地图矩形的格式化字符串。
 *  @param  rect    需要格式化的地图矩形。
 *  @return  包含地图矩形原点及尺寸的格式化字符串。
 */
UIKIT_STATIC_INLINE NSString *MBStringFromMapRect(MBMapRect rect) 
{
    return [NSString stringWithFormat:@"{%@, %@}", MBStringFromMapPoint(rect.origin), MBStringFromMapSize(rect.size)];
}

/**
 *
 *  @brief  返回表示两个矩形并集的矩形。
 *  @param  rect1   第一个矩形。
 *  @param  rect2   第二个矩形。
 *  @return 区域包含两个矩形及两个矩形间空间的矩形。
 *  @note  
 *      如果其中一个矩形无效，本方法返回另一个矩形。返回矩形的原点为两个矩形x及y的更小值。类似的，这个矩形的尺寸由x和y的最大
 *      值减去新原点的x和y值计算而得。
 */
UIKIT_EXTERN MBMapRect MBMapRectUnion(MBMapRect rect1, MBMapRect rect2);


/**
 *
 *  @brief  返回表示两个矩形交集的矩形。
 *  @param  rect1   第一个矩形。
 *  @param  rect2   第二个矩形。
 *  @return 表示两个矩形相交部分的矩形，如果没有交集则为MBMapRectNull。
 */
UIKIT_EXTERN MBMapRect MBMapRectIntersection(MBMapRect rect1, MBMapRect rect2);

/**
 *
 *  @brief  返回插入指定值的矩形。
 *  @param  rect    地图矩形。
 *  @param  dx  x轴两边减去的量（地图点）。
 *  @param  dy  y轴两边减去的量（地图点）。
 *  @return 插值矩形，如果原矩形无效则返回原矩形。
 */
UIKIT_EXTERN MBMapRect MBMapRectInset(MBMapRect rect, double dx, double dy);

/**
 *
 *  @brief  返回原点偏移了指定量的矩形。
 *  @param  rect    地图矩形。
 *  @param  dx  原点x轴的偏移量（地图点）。
 *  @param  dy  原点y轴的偏移量（地图点）。
 *  @return 偏移矩形，如果原矩形无效则返回原矩形。
 */
UIKIT_EXTERN MBMapRect MBMapRectOffset(MBMapRect rect, double dx, double dy);

/**
 *
 *  @brief  将指定的矩形分割为两个更小的矩形。
 *  @param  rect    将要分割的矩形。
 *  @param  slice   输出矩形移除的部分。
 *  @param  remainder   输出矩形剩余未被移除的部分。
 *  @param  amount  矩形指定边将要移除的数量。当这个值设为负数时，该值设为0。
 *  @param  edge    将要移除一定量的边。
 */
UIKIT_EXTERN void MBMapRectDivide(MBMapRect rect, MBMapRect *slice, MBMapRect *remainder, double amount, CGRectEdge edge);

/**
 *
 *  @brief  返回指示指定地图点是否在矩形中的布尔值。
 *  @param  rect    地图矩形。
 *  @param  point   检测的地图点。
 *  @return 如果矩形有效且非空，同时点位于矩形中返回YES,否则返回NO。
 *  @note
 *      当地图点在矩形的最小X或Y边是认为在矩形内。
 */
UIKIT_EXTERN BOOL MBMapRectContainsPoint(MBMapRect rect, MBMapPoint point);

/**
 *
 *  @brief  返回指示指定一个矩形是否包含另一个矩形的布尔值。
 *  @param  rect1   包含矩形。
 *  @param  rect2   可能包含在rect1中的矩形。
 *  @return 如果rect2有效且完全包含在rect1中返回YES,否则返回NO。
 */
UIKIT_EXTERN BOOL MBMapRectContainsRect(MBMapRect rect1, MBMapRect rect2);

/**
 *
 *  @brief  返回指示指定两个矩形是否相交。
 *  @param  rect1   第一个矩形。
 *  @param  rect2   第二个矩形。
 *  @return 如果rect1和rect2相交返回YES,如果不相交或者任意矩形无效则返回NO。
 *  @note
 *      如果只有边界部分相交则视为不相交。只有矩形相交区域的宽度和高度均大于0时视为真正的相交。
 */
UIKIT_EXTERN BOOL MBMapRectIntersectsRect(MBMapRect rect1, MBMapRect rect2);

/**
 *
 *  @brief  返回指定地图矩形对应的范围。
 *  @param  rect    想要获取对应区域的二维投影地图的地图矩形。
 *  @return 指定矩形对应经度、纬度及跨度的范围。
 */
UIKIT_EXTERN MBCoordinateRegion MBCoordinateRegionForMapRect(MBMapRect rect);

/**
 *
 *  @brief  返回只是指定矩形是否跨过了180子午线。
 *  @param  rect    地图矩形。
 *  @return 如果矩形跨过了180子午线返回YES,如果它整个在世界地图中则返回NO。
 */
 UIKIT_EXTERN BOOL MBMapRectSpans180thMeridian(MBMapRect rect);

/**
 *
 *  @brief  标准化处于指定矩形世界地图边界外的部分。
 *  @param  rect    地图矩形
 *  @note
 *      如果一个矩形处于180子午线，本方法分割出边界外的部分，再移除地图的另一边的部分，再返回这个矩形。
 */
UIKIT_EXTERN MBMapRect MBMapRectRemainder(MBMapRect rect);

/**
 *
 *  @brief   返回指定缩放等级下地图中路的宽度（屏幕点）。
 *  @param  zoomScale   当前应用在地图试图中的缩放因子。
 *  @return 道路的宽度，以屏幕点为单位计量。当需要描绘一条道路的路径时可以在绘制代码中使用这个返回值来设置线条的宽度。
 */
UIKIT_EXTERN CGFloat MBRoadWidthAtZoomScale(MBZoomScale zoomScale);
