//
//  MBMapView.h
//  MB"MBMapkit.h"
//
//  Copyright 2011 Mapbar Inc. All rights reserved.
//

#import <UIKit/UIKit.h>

#import "MBGeometry.h"
#import "MBAnnotationView.h"
#import "MBOverlayView.h"
#import "MBPolygon.h"
#import "MBPolyline.h"
#import "MBCircle.h"
#import "MBPolygonView.h"
#import "MBPolylineView.h"
#import "MBCircleView.h"
#import "MBUserLocation.h"
#import "MBReverseGeocoder.h"

enum {
    MBMapTypeStandard = 0,
};
typedef NSUInteger MBMapType;

@class MapRender;
@class MBUserLocation;
@class MBMapViewInternal;

@protocol MBMapViewDelegate;
/** @interface MBMapView
 *
 *  @brief
 *      MBMapView 对象提供可嵌入的地图界面，类似于地图应用提供的界面。你直接使用这个类来显示地图信息、在应用中控制地图的
 *      内容。你可以给定一个坐标作为地图的中心，指定想要显示的范围的大小，使用自定义的信息标注地图。
 *      \n在你初始化地图视图的时候，你需要指定地图显示的初始范围。你可以通过设置 region 属性来完成这个操作。范围是由中心点
 *      和水平与竖直距离（跨度）定义。跨度决定了在给定点的下多少地图可视，同时你可以用它设置缩放等级。指定一个大的跨度将导致
 *      用户看见一个大的地理区域对应一个低的缩放等级。指定一个小的跨度将导致用户看见一个更狭窄的地理区域对应一个高的缩放等级。
 *      \n除了通过编码设置跨度外， MBMapView 类支持许多标准接口用于改变地图的位置和缩放等级。特别的，地图视图支持双击和双
 *      指手势来滚动、放大和缩小地图。默认启用这些手势，但可以通过设置 scrollEnabled 和 zoomEnabled 属性来禁用手势。
 *      \n数据类型 MBMapPoint 、 MBMapSize 和 MBMapRect 用来指定地图上的位置和距离。你可以使用这些类型的数据来指定
 *      地图的可视范围和指定覆盖层的位置。
 *      \n你可以提供一个委托对象来获得地图视图运行的信息。地图视图调用你自定义委托的方法让委托知道地图状态的改变同时调整自定
 *      义标注的显示，
 *      下面的“标注地图”中将有更加详细的描述。委托的对象可以是你应用中的任何对象只要它遵从 MBMapViewDelegate 协议。参考
 *      MBMapViewDelegate 可以获得更多实现委托对象的信息。
 *  @section 标注地图
 *  @details 
 *      MBMapView类支持使用自定义的信息标注地图。因为地图可能有大量潜在的标注。地图视图区分管理标注数据的标注对象和在地图
 *      上展示这个数据的视图对象。
 *      \n标注对象可以上任何遵守 MBAnnotation 协议的对象。通常使用应用中数据模型已有的类来实现标注对象。这样你可以直接控制
 *      标注数据，而且还使它在地图视图中可用。每个标注对象包含了标注在地图的位置信息以及可以在弹出气泡中显示的描述信息。
 *      \n标注视图是一个 MBAnnotationView 类的实例，用于处理标注对象在地图中的表示。标注视图的负责用意义的方式展示标注数据。
 *      例如，地图应用使用一个大头针图标表示地图上的一个兴趣点。（Map Kit框架提供 MBPinAnnotationView 类作为应用中类似
 *      的标注）。你还可以创建的覆盖更大部分地图的标注视图。
 *      \n因为标注视图只有在屏幕内时才被需要， MBMapView 类采用了一个算法把未使用的标注视图放到队列。带有标识符的标注视图
 *      移出屏幕后将被地图视图内部的分离并放到队列。这个特征通过只让少量的标注视图保持在内存中及回收这些你有的视图的方式来提高
 *      内存的利用。同时这样还通过避免在地图滚动时创建新的视图来提高滚动的性能。
 *      \n在配置地图界面的时候，你需要立刻添加所有的标注对象。地图视图使用每个标注对象的坐标数据来决定对应的标注视图什么时候
 *      需要显示在屏幕上。当标注移进屏幕后，地图视图请求自己的委托来创建对应的标注视图。如果应用中有多种不同的标注，可以定义不
 *      同的标注视图类表示每种类型。
 *  @section 向地图中添加覆盖层
 *  @details
 *      你可以使用覆盖层显示覆盖地图较大区域的内容。覆盖层可以是任何遵守 MBOverlay 协议的对象。覆盖层对象是一个包含了许多用
 *      于指定覆盖层形状和大小以及在地图中位置的点的数据对象。覆盖层可以表示圆形、矩形和多分割的线条，还有简单或复杂的多边形。
 *      你还可以定义自己的覆盖层来表示其它的形状。
 *      \n标注视图是一个 MBOverlayView 类的实例，用于处理覆盖层对象在地图中的显示。覆盖层视图的任务是在地图上层绘制表示覆
 *      盖层的形状。例如，一个表示公交路线的覆盖层可能的覆盖层视图绘制了路线的路径同时沿着路线带有显示公交站的图标。Map Kit
 *      框架给几种标准的覆盖层对象定义了覆盖层视图，如果有需要你可以定义别的覆盖层视图。在配置地图界面的时候，你可以在任何时候
 *      添加覆盖层对象。地图视图根据每个覆盖层的数据决定对应的覆盖层视图什么时候需要显示在屏幕上。当覆盖层移进屏幕时，地图视图
 *      请求自己的委托创建对应的覆盖层视图。
 */
@interface MBMapView : UIView <UIScrollViewDelegate,CLLocationManagerDelegate,MBReverseGeocoderDelegate> {
@private
   MBMapViewInternal *_internal;
}

/**
 *  @brief  设置图吧家园注册码
 *  @param  registerCode    图吧家园注册码
 */
-(void)setRegisterCode:(NSString *)registerCode;

/**
 * 
 *  @brief  实例化地图视图。
 *  @param  frame  地图视图框架  
 *  @param  doLocal    是否使用本地缓存数据。
 *  @note   
 *      如果doLocal为YES，地图将缓存下载的地图数据。
 */ 
- (id)initWithFrame:(CGRect)frame withLocal:(BOOL)doLocal;

/**
 * 
 *  @brief  返回地图视图总的数据发送量。  
 *  @return 地图视图总的数据发送量。
 */ 
- (unsigned long long)totalBytesSended;

/**
 * 
 *  @brief  返回地图视图总的数据接收量。
 *  @return 地图视图总的数据接收量。
 */
- (unsigned long long)totalBytesReceived;

/** @property   delegate
 * 
 *  @brief  接收者的委托。
 *  @note   当加载地图数据和地图显示部分改变时地图视图发送信息给它的委托。委托还管理用于高亮地图上兴趣点的标注视图。
 *          \n委托需要实现MBMapViewDelegate协议中的方法。
 */
@property(nonatomic, assign) id<MBMapViewDelegate> delegate;

/** @property   region
 *
 *  @brief  地图视图当前显示的范围。
 *  @see    setRegion:animated:
 *  @see    centerCoordinate
 *  @note
 *      范围同时包含了地图上需要显示的经度和纬度的跨度。跨度值隐含的提供了地图的缩放值。
 *      显示区域越大，缩放等级越低。类似的，显示的范围越小，缩放等级越大。
 *      \n只改变范围的中心坐标仍可隐含的引起跨度的改变。这是因为在不同经度和纬度下跨度所代表的距离会改变，地图视图由于新位置
 *      的原因可能调整跨度。如果你希望在不改变缩放比的情况改变中心坐标，调用 setCenterCoordinate: 方法。
 *      \n改变本属性的数值将立刻更新地图视图。设置本属性的时候，地图可能调整新范围的值这样它才可以和地图的可视区域精确匹配。
 *      这样做很正常并且保证了区域属性总是和地图的可视部分对应。然而，这并不表明你调用本方法后可以立刻获取这个属性的数值，返回
 *      的值可能不和你设置的数值匹配。（你可以调用regionThatFits方法来确定实际在地图上设置的范围）。
 *      \n如果你希望动画改变范围，调用 setRegion:animated: 方法。
 */ 
@property (nonatomic) MBCoordinateRegion region;

/**  
 *
 *  @brief   改变当前可视区域，可选采用动画效果改变。
 *  @param  region  将在地图视图中显示的新范围  
 *  @param  animated    如果你想地图视图动画过渡到新的范围，指定YES；如果你想地图立即定位到指定的范围，指定NO。
 *  @see    region
 *  @see    setCenterCoordinate:animated:
 *  @note   
 *      只改变范围的中心坐标仍可潜在的引起跨度的改变。这是因为在不同经度和纬度下跨度所代表的距离会改变，地图视图可能由于新位置
 *      的原因调整跨度。如果你希望在不改变缩放等级的情况改变中心坐标，调用 setCenterCoordinate:animated: 方法。
 *      \n设置新范围的时候，地图可能调整范围的参数这样它才可以和地图的可视区域精确匹配。这样做很正常并且保证了区域属性总是和地
 *      图的可视部分对应。然而这并不表明你调用本方法后可以立刻获取这个属性的数值，返回的值可能不和你设置的数值匹配。（你可以调
 *      用 regionThatFits 方法来确定实际在地图上设置的范围）。
 */ 
- (void)setRegion:(MBCoordinateRegion)region animated:(BOOL)animated;

/** @property   centerCoordinate
 *
 *  @brief  地图视图中心的地图坐标。
 *  @see    setRegion:animated:
 *  @see    centerCoordinate
 *  @note   
 *      改变本属性的值将不改变当前缩放等级而定位到新的中心坐标。同时会更新 region 属性来反映新的中心坐标，而新的跨度将保持当
 *      前的缩放等级。改变本属性的值将立刻更新地图视图。
 *      \n如果你希望动画改变，调用 setCenterCoordinate:animated: 方法。
 */
@property(nonatomic) CLLocationCoordinate2D centerCoordinate;

/**  
 *
 *  @brief  改变地图的中心坐标，可选采用动画效果改变。
 *  @param  coordinate  地图的新中心坐标。  
 *  @param  animated    如果你想地图视图滚动到新的位置，指定YES；如果你想地图立即显示新的位置，指定NO。
 *  @see    region
 *  @see    centerCoordinate
 *  @note   
 *      改变中心坐标将在不改变当前缩放等级下定位到新的中心坐标。这还会更新 region 属性来反映新的中心坐标，同时新的跨度将保持
 *      现在的缩放等级。
 */ 
- (void)setCenterCoordinate:(CLLocationCoordinate2D)coordinate animated:(BOOL)animated;

/**
 *
 *  @brief  调整指定范围的长宽比以确保它和地图试图的框架匹配。
 *  @param  region  你想要调整跨度的初始化范围。
 *  @return 中心仍在地图中同一点但跨度已经被调整以适应地图视图的框架的范围。
 *  @note   
 *      你可以调用本方法在地图范围显示前将其标准化。本方法返回一个新的范围既包含了指定的矩形又和地图视图的框架整
 *      齐匹配。
 */
- (MBCoordinateRegion)regionThatFits:(MBCoordinateRegion)region;

/**
 *
 *  @brief  地图视图当前显示的区域。
 *  @note   
 *      本属性表示的基本信息和 region 属性相同，但是指定的一个地图矩形而不是一个范围。改变本属性的值将会立即更新地图视图。
 *      如果你希望动画改变，调用 setVisibleMapRect:animated: 方法。
 */
@property (nonatomic) MBMapRect visibleMapRect;

/** 
 *
 *  @brief  改变地图的当前可视部分，可选采用动画效果改变。
 *  @param  mapRect  将在地图视图中可视的地图矩形。  
 *  @param  animate    如果你想地图视图动画过渡到新的地图矩形，指定YES；如果你想地图立即定位到指定的矩形，指定NO。
 */ 
- (void)setVisibleMapRect:(MBMapRect)mapRect animated:(BOOL)animate;

/**  
 *
 *  @brief  调整指定地图矩形的长宽比以确保它和地图视图的框架匹配。
 *  @param  mapRect  你想要调整宽度和高度的初始地图矩形。
 *  @return 中心仍在地图中同一点但高度和宽度已经被调整以适应地图视图的框架的地图矩形。
 *  @note   
 *      你可以调用本方法在地图矩形其对应范围显示前将其标准化。本方法返回一个新的地图矩形既包含了指定的矩形又和地图视图的框架整
 *      齐匹配。
 */ 
- (MBMapRect)mapRectThatFits:(MBMapRect)mapRect;

/**
 *
 *  @brief   改变地图的当前可视部分，同时允许指定边缘额外的间隔。
 *  @param  mapRect 将在地图视图中可视的地图矩形。
 *  @param  insets  使指定矩形周围额外空间可视的数值（以屏幕点来计算）。
 *  @param  animate    如果你想地图视图动画过渡到新的地图矩形，指定YES；如果你想地图立即定位到指定的矩形，指定NO。
 */ 
- (void)setVisibleMapRect:(MBMapRect)mapRect edgePadding:(UIEdgeInsets)insets animated:(BOOL)animate;

/**
 *
 *  @brief  加上指定的插值调整指定地图矩形的长宽比。
 *  @param  mapRect  你想要调整宽度和高度的初始地图矩形。
 *  @param  insets  从地图视图框架实际边框返回的矩形需要插入的距离（屏幕点）。
 *  @return 中心仍在地图中同一点但高度和宽度已经被调整以适应减去插值后的地图视图的框架的地图矩形。
 */ 
- (MBMapRect)mapRectThatFits:(MBMapRect)mapRect edgePadding:(UIEdgeInsets)insets;

/**
 *
 *  @brief  转换地图坐标到指定视图中的点。
 *  @param  coordinate  你想找到对应点的地图坐标。
 *  @param  view    你想把指定地图坐标放到它的坐标系的视图。如果这个参数是nil,返回的点将指定在视窗坐标系。如果view不是nil,
 *                  它必须和地图视图隶属于同一视窗。
 *  @return 和指定经纬度值对应的点（在适当的视图或视窗坐标系中）。
 */ 
- (CGPoint)convertCoordinate:(CLLocationCoordinate2D)coordinate toPointToView:(UIView *)view;

/**   
 *  
 *  @brief  转换指定视图坐标系中的点到地图坐标。
 *  @param  point   你想要转换的点。
 *  @param  view    给point参数提供参考坐标系的视图。
 *  @return 指定点的地图坐标。
 */ 
- (CLLocationCoordinate2D)convertPoint:(CGPoint)point toCoordinateFromView:(UIView *)view;

/**
 *
 *  @brief  转换地图范围到指定视图中的矩形。
 *  @param  region  你想要找到对应视图矩形的地图范围。
 *  @param  view    该视图是你想把指定地图范围放到它的坐标系。如果这个参数是nil,返回的矩形将指定在视窗坐标系。如果\e view不
 *                  是nil,它必须和地图视图隶属于同一视窗。
 *  @return 返回指定地图范围对应的矩形。
 */ 
- (CGRect)convertRegion:(MBCoordinateRegion)region toRectToView:(UIView *)view;

/**
 *
 *  @brief  转换指定视图坐标系中的矩形到地图范围。 
 *  @param  rect    你想要转换的矩形。
 *  @param  view    给rect参数提供参考坐标系的视图。
 *  @return 指点视图矩形对应的地图范围。
 */ 
- (MBCoordinateRegion)convertRect:(CGRect)rect toRegionFromView:(UIView *)view;

/** @property   mapType
 *
 *  @brief  地图视图的数据类型。
 */
@property (nonatomic) MBMapType mapType;

/** @property   zoomEnabled
 *
 *  @brief  确定用户是否能够通过手势对地图进行缩放的布尔值。  
 *  @note
 *      本属性只控制用户和地图的交互。如果你改变本属性的值为NO，你仍可以通过编码改变 region 属性的值来改变缩放等级。
 *      \n默认本属性的值为YES。
 */
@property(nonatomic, getter=isZoomEnabled) BOOL zoomEnabled;

/** @property   scrollEnabled
 *    
 *  @brief  确定用户是否能够缩放地图的布尔值。
 *  @note   
 *      本属性只控制用户和地图的交互。如果你改变本属性的值为NO，你仍可以通过编码改变 region 属性的值来改变地图的位置。
 *      \n默认本属性的值为YES。
 */
@property(nonatomic, getter=isScrollEnabled) BOOL scrollEnabled;


/** @property   showsScale
 * 
 *   @brief  指示是否显示比例尺的布尔值。
 */
@property (nonatomic) BOOL showsScale;

/** @property   showsTraffic
 * 
 *   @brief  指示是否现实交通流量的布尔值。
 */
@property (nonatomic) BOOL showsTraffic;

/** @property   showsUserLocation
 *
 *  @brief  指示地图是否显示用户位置的布尔值。
 *  @note  
 *      本属性并不表示用户的位置在地图上是否实际上可视，只是指示地图视图是否允许显示。使用 userLocationVisible 属性来决
 *      定用户位置是否可视。本属性的默认值是NO。
 *      \n设置本属性为YES将引起地图视图调用Core Location框架寻找用户位置。一旦这个属性为YES,地图视图将一直跟踪用户的位置
 *      并且定期更新用户位置。
 */
@property (nonatomic) BOOL showsUserLocation;

/** @property   userLocation
 *
 *  @brief   代表用户当前位置的标注对象。
 */
@property (nonatomic, readonly) MBUserLocation *userLocation;

/** @property   userLocationVisible
 *
 *  @brief  指示设备的当前位置在地图视图中是否可视的布尔值。
 *  @note   
 *      本属性使用当前位置的水平精度决定用户位置是否可视。因此如果指定的坐标在屏幕外，但是包围这个坐标的矩形（由水平精度值定义
 *      ）有部分在屏幕内时本属性为YES
 *      \n如果用户的位置不能确定，属性的值为NO。
 */
@property (nonatomic, readonly, getter=isUserLocationVisible) BOOL userLocationVisible;

/** 
 *
 *  @brief  向地图视图中添加指定的标注。
 *  @param  annotation  添加到接收者的标注对象。 
 *  @see    addAnnotations:
 *  @see    removeAnnotation:
 *  @note   对象必须遵守 MBAnnotation 协议。地图视图将保留指定的对象。 
 */ 
- (void)addAnnotation:(id <MBAnnotation>)annotation;

/**  
 *
 *  @brief  向地图视图中添加一组标注。
 *  @param  annotations 一组标注对象。 
 *  @see    addAnnotations:
 *  @see    removeAnnotation:
 *  @note   对象必须遵守 MBAnnotation 协议。地图视图将保留指定的对象。 
 */ 
- (void)addAnnotations:(NSArray *)annotations;

/**
 *
 *  @brief  从地图视图中移除指定的标注。
 *  @param  annotation  将要移除的标注对象。对象必须遵守MBAnnotation协议。
 *  @see    removeAnnotations:
 *  @see    addAnnotation:
 *  @note   
 *      如果标注当前有关联的标注视图，并且这个视图有复用标识符，本方法移除标注视图同时为了以后的复用内在的把它放到复用队列中。
 *      你可以调用 dequeueReusableAnnotationViewWithIdentifier 方法取得队列中的标注视图（把它们和新的标注关联）。
 *      \n移除标注对象意味着彻底的将标注和地图视图解除关联，阻止它在地图中显示。因此，典型的当你在需要隐藏或删除一个给定标注
 *      时调用本方法。
 */ 
- (void)removeAnnotation:(id <MBAnnotation>)annotation;

/**
 *
 *  @brief  从地图视图中移除一组指定的标注对象。
 *  @param  annotations  一组将要移除的标注对象。每个对象必须遵守 MBAnnotation 协议。
 *  @see    removeAnnotations:
 *  @see    addAnnotation:
 *  @note
 *      如果数组中的任何标注当前有关联的标注视图，并且这个视图有复用标识符，本方法移除标注视图同时为了以后的复用内在的把它放到
 *      复用队列中。你可以调用 dequeueReusableAnnotationViewWithIdentifier 方法取得队列中的标注视图（把它们和新的
 *      标注关联）。
 *      \n移除标注对象意味着彻底的将标注和地图视图解除关联，阻止它在地图中显示。因此，典型的当你在需要隐藏或删除一个给定标注时
 *      调用本方法。
 */ 
- (void)removeAnnotations:(NSArray *)annotations;

/** @property   annotations
 *
 *  @brief  接收者关联的标注的完整列表。
 *  @see    addAnnotation:
 *  @see    addAnnotations:
 *  @see    removeAnnotation:
 *  @see    removeAnnotations:
 *  @note
 *      数组中的每个对象必须遵守 MBAnnotation 协议。如果地图视图没有关联任何标注，本属性的值为nil。
 */ 
@property (nonatomic, readonly) NSArray *annotations;

/** 
 *
 *  @brief  返回所有位于指定地图矩形内的标注。
 *  @param  mapRect 你想要搜索标注的地图部分。
 *  @return 返回一组mapRect内的所有标注对象。
 *  @note 
 *      本方法提供了一个快速的方式用于检索指定地图部分的标注对象。这个方法比你自己用 annotations 进行线性搜索快很多。
 */ 
- (NSSet *)annotationsInMapRect:(MBMapRect)mapRect;

/**  
 *
 *  @brief  如果有的话返回和指定标注关联的标注视图。
 *  @param  annotation  你想要其视图的标注对象。
 *  @return 标注视图。
 *  @note   如果视图尚未创建返回nil。如果标注不在可视地图范围因此没有关联的标注视图这样也可能返回nil。
 */ 
- (MBAnnotationView *)viewForAnnotation:(id <MBAnnotation>)annotation;

/**
 *
 *  @brief   通过标识符返回可复用的标注视图。
 *  @param  identifier  标识标注视图复用的字符串。这和你调用 initWithAnnotation:reuseIdentifier: 方法初始化标注
 *                      视图中的标识符是同一字符串。
 *  @return 含有指定标识符的标注视图，或者当复用队列中不存在这种对象时则为nil。
 *  @note   
 *      出于性能考虑，你一般需要在地图视图中复用 MBAnnotationView 对象。一旦标注视图移出屏幕，地图视图把它们移动到一个内
 *      部管理的复用队列。一旦新的标注移进屏幕，并且你的代码打算提供一个对应的标注视图，你总应该在创建一个新的视图前尝试重用
 *      一个存在的。重用在执行滚动这类关键操作时节省了时间和内存。
 */ 
- (MBAnnotationView *)dequeueReusableAnnotationViewWithIdentifier:(NSString *)identifier;

/**
 *
 *  @brief  选中指定的标注，并显示弹出气泡。
 *  @param  annotation  将要选中的标注对象。
 *  @param  animated    如果YES，弹出气泡将动画弹出。
 *  @return 含有指定标识符的标注视图，或者当复用队列中不存在这种对象时则为nil。
 *  @note
 *      如果指定的标注不在屏幕中，因此它也没有关联的标注视图，本方法没有任何作用。
 */ 
- (void)selectAnnotation:(id <MBAnnotation>)annotation animated:(BOOL)animated;

/**  
 *
 *  @brief  取消选择指定的标注同时隐藏它的弹出气泡。
 *  @param  annotation  将要取消选择的标注对象。
 *  @param  animated    如果YES,弹出气泡将动画移出屏幕。
 *  @return 含有指定标识符的标注视图，或者当复用队列中不存在这种对象时则为nil。
 */ 
- (void)deselectAnnotation:(id <MBAnnotation>)annotation animated:(BOOL)animated;

/** @property   selectedAnnotations
 *
 *  @brief  当前选中的标注。
 *  @note 
 *      给本属性分配一个新的数组时只会选中数组中的第一个标注。
 */
@property (nonatomic, copy) NSArray *selectedAnnotations;

/** @property   annotationVisibleRect
 *
 *  @brief  当前标注视图显示的可视矩形。
 */
@property (nonatomic, readonly) CGRect annotationVisibleRect;

/**  
 *
 *  @brief  向地图中添加一个覆盖层。
 *  @param  overlay 将要添加的覆盖层对象。对象必须遵守MBOverlay协议。 
 *  @note
 *      指定的对象将被添加到覆盖层对象列表的最后。添加一个覆盖层将引起地图视图监测覆盖层表示的范围。一旦覆盖层的边框和地图可视
 *      部分相交，地图视图添加相应的覆盖层视图到地图中。覆盖层视图由地图视图的委托对象调用 mapView:viewForOverlay: 方法
 *      提供。
 *      \n要从地图中移除一个覆盖层，你必须调用 removeOverlay: 方法来移除覆盖层对象。 
 */
- (void)addOverlay:(id <MBOverlay>)overlay;

/**
 *
 *  @brief  向地图中添加一组覆盖层。
 *  @param  overlays 一组对象，每个对象必须遵守MBOverlay协议。 
 *  @note   
 *      指定的对象将被添加到覆盖层对象列表的最后。添加一个覆盖层将引起地图视图监测覆盖层表示的范围。一旦覆盖层的边框和地图可视
 *      部分相交，地图视图添加相应的覆盖层视图到地图中。覆盖层视图由地图视图的委托对象调用 mapView:viewForOverlay: 方法
 *      提供。
 *      \n要从地图中移除一个覆盖层，你必须调用 removeOverlay: 方法来移除覆盖层对象。 
 */
- (void)addOverlays:(NSArray *)overlays;

/**  
 *
 *  @brief  从地图视图中移除一个覆盖层。
 *  @param  overlay  将要移除的覆盖层对象。
 *  @note
 *      移除一个覆盖层对象时如果它对应的视图当前正在显示还会移除它的视图。如果指定的对象并没有和地图视图关联，本方法将不执行
 *      任何操作。 
 */ 
- (void)removeOverlay:(id <MBOverlay>)overlay;

/**
 *
 *  @brief  从地图中移除一个或多个覆盖层。
 *  @param  overlays  一组对象，每个对象必须遵守 MBOverlay 协议。
 *  @note
 *      移除一个覆盖层对象时如果它对应的视图当前正在显示还会移除它的视图。如果指定的对象并没有和地图视图关联，本方法将不执行
 *      任何操作。 
 */ 
- (void)removeOverlays:(NSArray *)overlays;

/**
 *
 *  @brief  插入一个覆盖层到和地图关联的覆盖层对象列表中。
 *  @param  overlay 将要插入的覆盖层。
 *  @param  index   需要插入覆盖层对象的位置索引。如果这个数值大于overlays属性的对象数量，本方法将对象添加到数组的末尾。
 */ 
- (void)insertOverlay:(id <MBOverlay>)overlay atIndex:(NSUInteger)index;

/**
 *
 *  @brief  交换两个覆盖层的位置。
 *  @param  index1  第一个对象在overlays数组中的索引。
 *  @param  index2  第二个对象在overlays数组中的索引。
 *  @note 
 *      如果两个覆盖层都有对应的视图，视图的位置同时也更新。因此，当覆盖层视图显示在地图视图时交换它们同时将影响它们的z轴顺序。
 */ 
- (void)exchangeOverlayAtIndex:(NSUInteger)index1 withOverlayAtIndex:(NSUInteger)index2;

/**
 *
 *  @brief  插入一个覆盖层到另一个的上面。
 *  @param  overlay 将要插入的覆盖层。
 *  @param  sibling 一个overlays数组中存在的对象。这个对象必须存在于数组中并且非空。
 *  @note
 *      本方法添加\e overlay到地图视图，把它放到\e overlays数组中指定的sibling对象相对的位置。这个位置导致\e overlay
 *      关联的视图将被显示在\e sibling关联视图的上面。
 */ 
- (void)insertOverlay:(id <MBOverlay>)overlay aboveOverlay:(id <MBOverlay>)sibling;

/**
 *
 *  @brief   插入一个覆盖层到另一个的下面。
 *  @param  overlay 将要插入的覆盖层。
 *  @param  sibling 一个overlays数组中存在的对象。这个对象必须存在于数组中并且非空。
 *  @note
 *      本方法添加\e　overlay到地图视图，把它放到overlays数组中指定的\e sibling对象相对的位置。这个位置导致\e overlay
 *      关联的视图将被显示在\e sibling关联视图的下面。
 */ 
- (void)insertOverlay:(id <MBOverlay>)overlay belowOverlay:(id <MBOverlay>)sibling;

/**
 *
 *  @brief  当前和地图视图关联的覆盖层。
 *  @note
 *      数组中的对象必须遵守MBOverlay协议。如果地图视图没有关联任何覆盖层，本属性的值为一个空数组。
 *      \n如果两个覆盖层的范围相交，本数组中对象的顺序决定了对应的覆盖层视图在地图上显示的位置。数组中前面的覆盖层对象将放在
 *      数组中后面出现的覆盖层下面。因此，索引为0的覆盖层显示在索引为1的覆盖层的下面。
 */
@property (nonatomic, readonly) NSArray *overlays;

/**
 *
 *  @brief  如果有的话返回和指定覆盖层相关的视图。
 *  @param  overlay 你想要其视图的标注覆盖层对象。
 *  @return 覆盖层关联的视图，覆盖层不在屏幕内时返回nil。
 */ 
- (MBOverlayView *)viewForOverlay:(id <MBOverlay>)overlay;

@end

/** @protocol MBMapViewDelegate
 *
 *  @brief
 *      MBMapViewDelegate协议定义了一组可选的方法让你可以接收到地图相关的更新信息。因为很多地图操作需要MBMapView类异步
 *      加载数据，当指定的操作完成后地图视图调用这些方法通知你的应用。地图视图也会调用这些方法来请求标注和覆盖层的视图同时管理
 *      这些视图的交互。在释放MBMapView对象前，如果你设置了委托，记得把对象的委托属性设为nil。在你调用销毁地图视图的dealloc
 *      函数时可以执行前面所说的操作。
 */
@protocol MBMapViewDelegate <NSObject>

@optional

/**
 *
 *  @brief  通知委托地图视图显示的范围将要改变。
 *  @param  mapView 可视范围将要改变的地图视图。
 *  @param  animated    如果YES,将动画变到新范围。如果NO,改变将立刻生效。
 *  @note
 *      任何当前可视范围改变时本方法都会被调用。滚动的时候本方法会被调用很多次来报告地图位置的更新。因此本方法的实现应尽可能的
 *      轻量以避免影响滚动效率。
 */ 
- (void)mapView:(MBMapView *)mapView regionWillChangeAnimated:(BOOL)animated;

/**
 *
 *  @brief  通知委托地图显示的范围刚改变。
 *  @param  mapView 可视范围已改变的地图视图。
 *  @param  animated    如果YES,到新范围的变化是动画的。
 *  @note
 *      任何当前可视范围改变时本方法都会被调用。滚动的时候本方法会被调用很多次来报告地图位置的更新。因此本方法的实现应尽可能的
 *      轻量以避免影响滚动效率。
 */ 
- (void)mapView:(MBMapView *)mapView regionDidChangeAnimated:(BOOL)animated;

/** 
 *
 *  @brief  通知委托指定的地图视图将要请求一些地图数据。
 *  @param  mapView 已经开始加载数据的地图视图。
 *  @note
 *      任何需要从服务器下载一组新地图块的时候本方法将被调用。这一般发生在你通过移动缩放内容来浏览地图的时候。你可以调用本方法
 *      来记录地图视图加载数据所需的时间。
 */ 
- (void)mapViewWillStartLoadingMap:(MBMapView *)mapView;

/** 
 *
 *  @brief  通知委托指定的地图视图成功的加载了需要的地图数据。
 *  @param  mapView 已经开始加载操作的地图视图。
 *  @note
 *      当前请求相关的地图块已被加载的时候本方法将被调用。当新的可视区域滚到视图但没有这块区域的地图块时将请求这些地图块。当前
 *      地图中不可视的部分地图块也可能被请求。例如，地图视图可能需要立刻加载可视范围周围一圈的地图块来处理用户的移动操作。。
 */ 
- (void)mapViewDidFinishLoadingMap:(MBMapView *)mapView;

/** 
 *
 *  @brief  通知委托指定的地图视图不能加载地图数据。
 *  @param  mapView 已经开始加载操作的地图视图。
 *  @param  error   地图数据不能加载的原因。
 *  @note
 *      在设备不能访问网络或者因为某些原因不能加载地图数据的情况下本方法会被调用。在先前的地图块请求仍出于等待的时候又有另外一
 *      个地图块请求进来了也可能调用本方法。你可以调用这个信息通知用户地图数据不可用。
 */ 
- (void)mapViewDidFailLoadingMap:(MBMapView *)mapView withError:(NSError *)error;

/** 
 *
 *  @brief  返回与指定标注对象关联的视图。
 *  @param  mapView 请求标注视图的地图视图。
 *  @param  annotation  将被显示的标注对象。除了你自定义的标注，这个物体可能是表示用户当前位置的MBUserLocation对象。
 *  @return 表示自定义标注的标注视图，或者你希望显示一个标准标注视图时返回nil.
 *  @note
 *      你应该调用MB"MBMapkit.h"类中的 dequeueReusableAnnotationViewWithIdentifier 方法来查看是否已有一个所请求的类型
 *      标注视图，而不是每次调用本方法时都创建一个新的视图。如果一个真的存在，你需要更新视图来对应指定的标注的属性然后返回它。
 *      如果想要的类型的视图不存在，你应该创建一个，用需要的标注数据配置然后返回它。
 *      \n如果这个标注参数的对象是一个 MBUserLocation 的实例，你可以提供一个自定义的视图作为用户位置的标志。若要显示默认
 *      系统视图来表示用户坐标，则返回nil.
 *      \n如果你不实现本方法，或者你的实现中除了用户位置标注外的标注返回nil，地图视图将使用标准大头针标注视图。
 */ 
- (MBAnnotationView *)mapView:(MBMapView *)mapView viewForAnnotation:(id <MBAnnotation>)annotation;

/**
 *
 *  @brief   通知委托一个或多个标注视图被添加到地图。
 *  @param  mapView 添加了标注视图的地图视图。
 *  @param  views   一组被添加的表示视图的MBAnnotaitionView对象。
 *  @note 
 *      本方法调用的时候，指定的视图已经添加到了地图。
 */ 
- (void)mapView:(MBMapView *)mapView didAddAnnotationViews:(NSArray *)views;

/**
 *
 *  @brief  通知委托用户点击了一个标注视图的附加按钮。
 *  @param  mapView 包含指定标注视图的地图视图。
 *  @param  view    按钮被点击的标注视图。
 *  @param  control 被点击的控件。
 *  @note
 *      附加视图包含自定义内容并且放置在标注标题文本的两边。如果你指定的视图是UIControl类的派生，任何用户点击你的视图时地图
 *      视图将调用本方法。你可以调用本方法来响应与控件关联的任何操作。例如，如果你的控件显示标注额外的信息，你应该调用本方法来
 *      展示一个包含那个信息的标准面板。
 *      \n如果你自定义的视图不是UIControl类的派生,地图视图将不会调用本方法。
 */ 
- (void)mapView:(MBMapView *)mapView annotationView:(MBAnnotationView *)view calloutAccessoryControlTapped:(UIControl *)control;

/**
 *
 *  @brief  通知委托其中一个标注视图被选择了。
 *  @param  mapView 包含指定标注视图的地图视图。
 *  @param  view    被选择的标注视图。
 *  @note
 *      你可以调用本方法来跟踪标注视图的选择状态的改变。
 */ 
- (void)mapView:(MBMapView *)mapView didSelectAnnotationView:(MBAnnotationView *)view;

/**
 *
 *  @brief  通知委托其中一个标注视图被取消选择了。
 *  @param  mapView 包含指定标注视图的地图视图。
 *  @param  view    被取消选择的标注视图。
 *  @note
 *      你可以调用本方法来跟踪标注视图的选择状态的改变。
 */ 
- (void)mapView:(MBMapView *)mapView didDeselectAnnotationView:(MBAnnotationView *)view;

/** 
 *
 *  @brief  通知委托地图视图将要开始跟踪用户的位置。
 *  @param  mapView 正在跟踪用户位置的地图视图。
 *  @note
 *      当showsUserLocation属性变为YES时，本方法将被调用。
 */ 
- (void)mapViewWillStartLocatingUser:(MBMapView *)mapView;

/**
 *
 *  @brief  通知委托地图视图停止跟踪用户的位置。
 *  @param  mapView 停止跟踪用户位置的地图视图。
 *  @note
 *      当showsUserLocation属性变为NO时，本方法将被调用。
 */ 
- (void)mapViewDidStopLocatingUser:(MBMapView *)mapView;

/** 
 *
 *  @brief  通知委托用户的位置已更新。
 *  @param  mapView 正在跟踪用户位置的地图视图。
 *  @param  userLocation    表示用户最新位置的位置对象。
 *  @note
 *      showUserLocation属性被设为YES时，任何地图视图接收到位置更新时本方法都会被调用。
 *      \n应用在后台运行时本方法不会被调用。如果你希望后台运行是仍能收到位置更新，你必须使用Core Location框架。
 */ 
- (void)mapView:(MBMapView *)mapView didUpdateUserLocation:(MBUserLocation *)userLocation;

/** 
 *
 *  @brief  通知委托尝试定位用户的位置失败。
 *  @param  mapView 正在跟踪用户位置的地图视图。
 *  @param  error   包含为什么定位跟踪失败原因的错误对象。
 */ 
- (void)mapView:(MBMapView *)mapView didFailToLocateUserWithError:(NSError *)error;

/**
 *
 *  @brief  通知委托其中一个标注视图的状态改变了。
 *  @param  mapView 包含指定标注视图的地图视图。
 *  @param  view    拖动状态改变了的标注视图。
 *  @param  newState    标注视图的新状态。
 *  @param  oldState    标注视图先前的状态。
 *  @note 
 *      拖动状态一般回应用户和标注视图的交互而改变。然而，标注视图自己也会负责状态改变状态。
 */ 
- (void)mapView:(MBMapView *)mapView annotationView:(MBAnnotationView *)view didChangeDragState:(MBAnnotationViewDragState)newState 
   fromOldState:(MBAnnotationViewDragState)oldState;

/** 
 *
 *  @brief  当需要显示指定覆盖层对象时向委托请求覆盖层视图。
 *  @param  mapView 请求覆盖层视图的地图视图。
 *  @param  overlay 将被显示的覆盖层对象。
 *  @return 表示地图中指定覆盖层的视图。如果你返回nil，没有表示指定覆盖层对象的视图会被显示。
 *  @note  
 *      如果你支持显示覆盖层，你必须实现本方法同时提供覆盖层对象的视图。
 */ 
- (MBOverlayView *)mapView:(MBMapView *)mapView viewForOverlay:(id <MBOverlay>)overlay;

/**
 *
 *  @brief  通知委托一个或多个覆盖层视图被添加到地图。
 *  @param  mapView 添添加了覆盖层的地图。
 *  @param  overlayViews   一组被添加的表示视图的 MBOverlayView 对象。
 *  @note 
 *      本方法调用的时候，指定的视图已经添加到了地图。
 */ 
- (void)mapView:(MBMapView *)mapView didAddOverlayViews:(NSArray *)overlayViews;

@end