//
//  MBReverseGeocoder.h
//  MBMapKit
//
//  Copyright 2011 Mapbar Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>

@class MBPlacemark;
@class MBReverseGeocoderInternal;
@protocol MBReverseGeocoderDelegate;
/** @interface MBReverseGeocoder
 *
 *  @brief
 *      MBReverGeocoder类提供服务转化地图坐标点（由经度、维度指定）到那个坐标点信息，例如国家、城市或街道。逆地址编码器
 *      对象是一次执行对象连同基于联网的地图服务用于查找指定精度值的兴趣点信息。
 */
@interface MBReverseGeocoder : NSObject{
    MBReverseGeocoderInternal *_internal;
}

/**
 *
 *  @brief  使用指定的坐标值初始化逆地址编码器。
 *  @param  coordinate  你想要取得兴趣点信息的地图坐标。
 *  @return 初始化的 MBReverseGeocoder 对象.
 */
- (id)initWithCoordinate:(CLLocationCoordinate2D)coordinate;

/**
 *
 *  @brief  异步开始逆地址编码进程。
 *  @note
 *      开始逆地址编码进程后本方法只能调用一次。本方法异步给地图服务提交坐标值。一旦进程结束，结果就会传递给关联的委托对象。
 */
- (void)start;

/**
 *
 *  @brief  取消一个即将发生的逆地址编码请求。
 *  @note   
 *      你可以调用本方法来取消一个即将发生的请求，同时释放与那个请求相关的资源。如果请求已经返回或者还没有开始，调用本方法将
 *      没有影响。
 */
- (void)cancel;

/** @property   delegate
 *
 *  @brief  逆地址编码器的委托对象。
 *  @note   
 *      逆地址编码器对象发送有关成功（或不成功）获取兴趣点数据的消息给它的委托。你必须提供一个委托对象用于接受这个数据。详细信
 *      息请参考MBReverseGeocoderDelegate类。

 */
@property (nonatomic, assign) id<MBReverseGeocoderDelegate> delegate;

/** @property   coordinate
 *
 *  @brief  你想要取得兴趣点信息的坐标。
 */
@property (nonatomic, readonly) CLLocationCoordinate2D coordinate;

/**
 *
 *  @brief  逆地址编码操作的结果。
 *  @note   本属性默认值为nil。一个成功的逆地址编码操作后，它被设为生成的兴趣点对象。
 */
@property (nonatomic, readonly) MBPlacemark *placemark;

/** 
 *
 *  @brief  表示接收者是否在逆地址编码坐标的过程中的布尔值。
 *  @note  
 *      当这个进程还在进行中是本属性为YES,当进程结束或还未初始化时，本属性为NO。
 */
@property (nonatomic, readonly, getter=isQuerying) BOOL querying;

@end

/** @protocol MBReverseGeocoderDelegate
 *
 *  @brief
 *      MBReverseGeocoderDelegate协议定义了从 MBReverseGeocoder 对象接收消息的接口。你使用本借口来接收给定坐标的兴趣
 *      点信息或是逆地址编码过程中发生的任何错误信息。
 *      \n委托必须实现本协议的所有方法。
 */
@protocol MBReverseGeocoderDelegate <NSObject>
@required

/**
 *
 *  @brief  告知委托逆地址编码器对象成功的获取了对应坐标的兴趣点信息。
 *  @param  geocoder    成功完成请求的逆地址编码器对象。
 *  @param  placemark   包含兴趣点数据的对象。
 *  @note  
 *      你可以获取来自逆地址编码器对象或兴趣点对象的关联请求的地图坐标，只要它遵守MBAnnotation协议。
 */
- (void)reverseGeocoder:(MBReverseGeocoder *)geocoder didFindPlacemark:(MBPlacemark *)placemark;

/**
 *
 *  @brief   告知委托逆地址编码器对象未能获取对应坐标的兴趣点信息。
 *  @param  geocoder    未能完成请求的逆地址编码器对象。
 *  @param  error   表示请求未能成功原因的错误对象。
 */
- (void)reverseGeocoder:(MBReverseGeocoder *)geocoder didFailWithError:(NSError *)error;

@end
